//-----------------------------------------------------------------------------
/*
 * Description: This file contains the class AxPluginSetup
 *
 * See the file README in the main directory for a description of
 * this software, copyright information, and how to reach the author.
 *
 * Author: alex
 * Date:   04.11.2004
 *
 * Last modfied:
 *   $Author: alex $
 *   $Date: 2006-03-02 00:42:52 +0100 (Do, 02 Mär 2006) $
 *   $Revision: 18 $
 */
//-----------------------------------------------------------------------------
#ifndef __AxPluginSetup_H__
#define __AxPluginSetup_H__

//-----------------------------------------------------------------------------
//     includes
//-----------------------------------------------------------------------------

//----- swap-workaround -------------------------------------------------------
#include "AxPluginGlobals.h"

//----- qt --------------------------------------------------------------------
//----- CORBA -----------------------------------------------------------------
//----- C++ -------------------------------------------------------------------
#include <string>
#include <vector>

//----- C ---------------------------------------------------------------------
//----- AxLib -----------------------------------------------------------------
//----- local -----------------------------------------------------------------
//----- vdr -------------------------------------------------------------------
//----- local -----------------------------------------------------------------

//=============================================================================
//     forward declarations
//=============================================================================
class cPlugin;

//=============================================================================
//     typedefs
//=============================================================================

//=============================================================================
//     class AxPluginSettings
//=============================================================================
/** Small class to hold the common settings of the plugin
 */
class AxPluginSettings
{
  public:
    //-------------------------------------------------------------------------
    //     AxPluginSettings()
    //-------------------------------------------------------------------------
    AxPluginSettings();

    //-------------------------------------------------------------------------
    //     parseSetup()
    //-------------------------------------------------------------------------
    bool parseSetup(const char *theName, const char *theValue);

    //-------------------------------------------------------------------------
    //     storeSetup()
    //-------------------------------------------------------------------------
    void storeSetup(cPlugin *thePlugin);

    //-------------------------------------------------------------------------
    //     attributes (public for easy access)
    //-------------------------------------------------------------------------
    int  ActivateOnStart  ;
    int  HideMainMenuEntry;
    int  MainMenuAction   ;
    int  ShowStatusMsg    ;

}; // class AxPluginSettings

//=============================================================================
//     class AxOSDPreset
//=============================================================================
/** Small class to hold one OSD-preset
 */
class AxOSDPreset
{
  public:
    //-------------------------------------------------------------------------
    //     AxOSDPreset()
    //-------------------------------------------------------------------------
    AxOSDPreset();

    //-------------------------------------------------------------------------
    //     activate()
    //-------------------------------------------------------------------------
    /** activates the preset if valid
     */
    void activate();

    //-------------------------------------------------------------------------
    //     invalidate()
    //-------------------------------------------------------------------------
    void invalidate();

    //-------------------------------------------------------------------------
    //     isEmpty()
    //-------------------------------------------------------------------------
    /** checks whether the preset invalidated -> contains only initial values
     */
    bool isEmpty() const;

    //-------------------------------------------------------------------------
    //     isValid()
    //-------------------------------------------------------------------------
    /** checks whether the preset is valid
     */
    bool isValid() const;

    //-------------------------------------------------------------------------
    //     isActive()
    //-------------------------------------------------------------------------
    /** checks whether the current OSD-Settings match this preset
     */
    bool isActive() const;

    //-------------------------------------------------------------------------
    //     useCurrent()
    //-------------------------------------------------------------------------
    void useCurrent();

    //-------------------------------------------------------------------------
    //     dump()
    //-------------------------------------------------------------------------
    void dump() const;

    //-------------------------------------------------------------------------
    //     getInfoString()
    //-------------------------------------------------------------------------
    std::string getInfoString() const;

    //-------------------------------------------------------------------------
    //     attributes (public for easy access)
    //-------------------------------------------------------------------------
    std::string    Name;      ///< Name of the preset
    int            OSDLeft;   ///< 0            <= left   <= ???
    int            OSDTop;    ///< 0            <= top    <= ???
    int            OSDWidth;  ///< MINOSDWIDTH  <= width  <= MAXOSDWIDTH
    int            OSDHeight; ///< MINOSDHEIGHT <= height <= MAXOSDHEIGHT

}; // class AxOSDPreset

//=============================================================================
//     typedefs
//=============================================================================
typedef std::vector<AxOSDPreset> AxOSDPresetCltn;

//=============================================================================
//     class AxPluginSetup
//=============================================================================
/** Class that holds the common settings and several OSD-presets
 */
class AxPluginSetup
{
  public:
    //-------------------------------------------------------------------------
    //     AxPluginSetup()
    //-------------------------------------------------------------------------
    AxPluginSetup();

    //-------------------------------------------------------------------------
    //     ~AxPluginSetup()
    //-------------------------------------------------------------------------
    ~AxPluginSetup();

    //-------------------------------------------------------------------------
    //     loadPresets()
    //-------------------------------------------------------------------------
    /** cPlugin::ConfigDirectory(myPlugin->Name())
     */
    bool loadPresets(const std::string &theConfigFolder);

    //-------------------------------------------------------------------------
    //     savePresets()
    //-------------------------------------------------------------------------
    /** cPlugin::ConfigDirectory(myPlugin->Name())
     */
    bool savePresets(const std::string &theConfigFolder);

    //-------------------------------------------------------------------------
    //     guessActive()
    //-------------------------------------------------------------------------
    /** guess which preset is currently active by comparing presets whith
     *  current settings. myCurrentPreset is checked first. If myCurrentPreset
     *  isn't active now myCurrentPreset my be modified, if another preset
     *  matches the current OSD-settings
     *
     *  @returns the Index or -1 if none found
     */
    int guessActive();

    //-------------------------------------------------------------------------
    //     getCurrentPreset()
    //-------------------------------------------------------------------------
    int getCurrentPreset() const { return myCurrentPreset; }

    //-------------------------------------------------------------------------
    //     getNext()
    //-------------------------------------------------------------------------
    /** returns the index of the next preset, which will get activated by
     *  a call to activateNext()
     *
     *  @returns the Index or -1 if none found
     */
    int getNext() const;

    //-------------------------------------------------------------------------
    //     activateNext()
    //-------------------------------------------------------------------------
    void activateNext(bool fShowMsg, bool fShowWarnMsg = true);

    //-------------------------------------------------------------------------
    //     activatePreset()
    //-------------------------------------------------------------------------
    int activatePreset(int theIndex, bool fShowMsg, bool fShowWarnMsg = true);

    //-------------------------------------------------------------------------
    //     findPreset()
    //-------------------------------------------------------------------------
    /** returns the index of the preset with the given name or
     *  a value >= Presets.size() if not present
     */
    int findPreset(const std::string &theName) const;

    //-------------------------------------------------------------------------
    //     attributes (public for easy access)
    //-------------------------------------------------------------------------
    AxPluginSettings Settings;
    AxOSDPresetCltn  Presets ;

  private:
    //-------------------------------------------------------------------------
    //     attributes (private)
    //-------------------------------------------------------------------------
    int myCurrentPreset;

}; // class AxPluginSetup

#endif

